use crate::{config::DriveConfig, linux::run_command};

// TODO : Add support for using entire block device

/// Format the drives with the given config
pub fn format_drives(conf: &DriveConfig) {
    // TODO : Safety checks !!!

    // EFI (BOOT)
    run_command(&["mkfs.vfat", "-F", "32", conf.boot.as_str()], None, false);

    // ROOT
    if let Some(pass) = &conf.encryption {
        run_command(
            &["cryptsetup", "-q", "luksFormat", conf.root.as_str()],
            Some(&format!("{pass}\n")),
            false,
        );

        run_command(
            &["cryptsetup", "open", conf.root.as_str(), "root"],
            Some(&format!("{pass}\n")),
            false,
        );

        run_command(&["mkfs.ext4", "/dev/mapper/root"], None, false);
    } else {
        run_command(&["mkfs.ext4", conf.root.as_str()], None, false);
    }
}

// MOUNT

/// Mount the drives at `/mnt`
pub fn mount_drives(conf: &DriveConfig) {
    if conf.encryption.is_some() {
        run_command(&["mount", "/dev/mapper/root", "/mnt"], None, false);
    } else {
        run_command(&["mount", conf.root.as_str(), "/mnt"], None, false);
    }

    run_command(
        &[
            "mount",
            "--mkdir",
            conf.boot.as_str(),
            "/mnt/boot",
            "-o",
            "rw,nosuid,nodev,noatime,fmask=0137,dmask=0027",
        ],
        None,
        false,
    );
}
