// TPM Unlock

use yansi::{Color, Paint};

use crate::{
    config::DriveConfig,
    linux::{arch_chroot, install_file, run_command},
    pkg::install_pkgs,
};

/// Setup TPM Unlock and a recovery key for the root drive
pub fn setup_tpm_unlock(conf: &DriveConfig) {
    install_pkgs(&["tpm2-tools"]);

    // systemd-cryptenroll --tpm2-device=list

    // Recovery Key
    let recovery_key = arch_chroot(
        &["systemd-cryptenroll", "--recovery-key", &conf.root],
        Some(&format!("{}\n", conf.encryption.as_ref().unwrap())),
        false,
    )
    .0;
    install_file("/mnt/root/recovery.key", &recovery_key, 0o400);

    arch_chroot(
        &[
            "systemd-cryptenroll",
            "--tpm2-device=auto",
            &conf.root,
            "--tpm2-pcrs=7",
        ],
        Some(&format!("{}\n", conf.encryption.as_ref().unwrap())),
        false,
    );
}

// SECURE BOOT

/// Setup Secure Boot on the system
pub fn setup_secure_boot() {
    let (stdout, _) = run_command(&["sbctl", "status"], None, false);
    let binding = stdout.lines().collect::<Vec<&str>>();
    let status = binding.get(1).unwrap();

    if !status.contains("Setup Mode") || !status.contains("Enabled") {
        println!(
            "{}",
            "[!] Secure Boot is not in Setup Mode".paint(Color::Red)
        );
        std::process::exit(1);
    }

    install_pkgs(&["sbctl", "sbsigntools"]);

    arch_chroot(&["sbctl", "create-keys"], None, false);

    arch_chroot(&["sbctl", "enroll-keys", "--microsoft"], None, false);

    sb_sign("/boot/EFI/Linux/arch-linux.efi");
    sb_sign("/boot/EFI/Linux/arch-linux-fallback.efi");
    sb_sign("/boot/EFI/systemd/systemd-bootx64.efi");
    sb_sign("/boot/EFI/Boot/bootx64.efi");

    arch_chroot(&["sbctl", "verify"], None, false);
}

pub fn sb_sign(file: &str) {
    arch_chroot(&["sbctl", "sign", "-s", file], None, false);
}
